package YClass;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.Vector;

/**
 * A panel that displays Windows 95-style folder tabs that displays 
 * discrete components when a tab is selected. 
 */
public class YTabPanel extends Panel
{

/**
* Constructs a new FolderTabPanel.
*/
public YTabPanel()
    {
        labels = new Vector();
        panels = new Vector();
        tabHeight = 21;
        tabWidthBuffer = 13;
        p_width = 300;
        p_height = 50;
        insets = new Insets(14 + tabHeight, 14, 14, 14);
        setLayout(new BorderLayout());
        enableEvents(16L);
    }
  /**
   * Adds a tab to the TabPanel. 
   * @param panel the panel to display when the tab is selected
   * @param s - the tab label
   */
    public void addPanel(Panel panel, String s)
    {
        panels.addElement(panel);
        labels.addElement(s);
        add(panel);
        Dimension dimension = panel.getPreferredSize();
        p_height = Math.max(dimension.height, p_height);
        panel.setBounds(1, 22, getSize().width - 3, getSize().height - 3 - tabHeight);
        panel.setVisible(false);
        if(panels.size() == 1)
            setPanel(s);
    }
  /**
   * Remove a tab to the TabPanel. 
   * @param s the tab label
   */
    public void removePanel(String s)
    {
        int i = labels.indexOf(s);
        if(i == -1)
            throw new IllegalArgumentException(s + " is not a panel in this YTabPanel");
        labels.removeElementAt(i);
        panels.removeElementAt(i);
        if(s.equals(selectedPanel))
        {
            if(labels.size() > 0)
            {
                setPanel((String)labels.elementAt(0));
                return;
            }
            selectedPanel = null;
            repaint();
        }
    }

    public void next()
    {
        int i = labels.indexOf(selectedPanel);
        if(i == -1)
            throw new IllegalArgumentException(" is not a panel in this YTabPanel");
        if(i + 1 < labels.size())
            setPanel((String)labels.elementAt(i + 1));
    }

    public void previous()
    {
        int i = labels.indexOf(selectedPanel);
        if(i == -1)
            throw new IllegalArgumentException(" is not a panel in this YTabPanel");
        if(i > 0)
            setPanel((String)labels.elementAt(i - 1));
    }
  /**
   * Sets the TabPanel's active tab. 
   * @param s the label of the tab to select.
   */
    public void setPanel(String s)
    {
        int i = labels.indexOf(s);
        if(i == -1)
            throw new IllegalArgumentException(s + " is not a panel in this YTabPanel");
        selectedPanel = s;
        Panel panel = (Panel)panels.elementAt(i);
        for(int j = 0; j < labels.size(); j++)
            if(j != i)
                ((Panel)panels.elementAt(j)).setVisible(false);

        doLayout();
        panel.setBounds(getInsets().left, getInsets().top, getSize().width - getInsets().left - getInsets().right, getSize().height - getInsets().top - getInsets().bottom);
        panel.doLayout();
        panel.setVisible(true);
    }
  /**
   * Performs a layout() that shapes the current tab into the TabPanel. 
   */
    public void doLayout()
    {
        super.doLayout();
        for(int i = 0; i < labels.size(); i++)
        {
            Panel panel = (Panel)panels.elementAt(i);
            panel.setBounds(getInsets().left, getInsets().top, getSize().width - getInsets().left - getInsets().right, getSize().height - getInsets().top - getInsets().bottom);
            panel.doLayout();
        }

    }

    public void setSize(int i, int j)
    {
        super.setSize(i, j);
    }
  /**
   * Returns the label of the selected tab. 
   */
    public String getSelectedPanelLabel()
    {
        return selectedPanel;
    }
  /**
   * Returns the panel of the selected tab. 
   */
    public Panel getSelectedPanel()
    {
        return (Panel)panels.elementAt(labels.indexOf(selectedPanel));
    }
  /**
   * Returns the border within the TabPanel to display 
   * between the rectangle and the contained components.  
   */
    public Insets getInsets()
    {
        return insets;
    }
  /**
   * Sets the border within the TabPanel to display 
   * between the rectangle and the contained components.  
   */
    public void setInsets(Insets insets1)
    {
        insets = insets1;
    }
  /**
   * Returns the height of the tabs.
   */
    public int getTabHeight()
    {
        return tabHeight;
    }
  /**
   * Set the height of the tabs.
   */
    public void setTabHeight(int i)
    {
        tabHeight = i;
    }
  /**
   * Returns the margin to the left and right of the tab text. 
   */
    public int getTabWidthBuffer()
    {
        return tabWidthBuffer;
    }
  /**
   * Set the margin to the left and right of the tab text. 
   */
    public void setTabWidthBuffer(int i)
    {
        tabWidthBuffer = i;
    }
    /**
     * Gets the preferred size of this component.
     * @return A dimension object indicating this component's preferred size.
     * @see #getMinimumSize
     */  
    public Dimension getPreferredSize()
    {
        return new Dimension(p_width, p_height + tabHeight + 5);
    }
    /**
     * Gets the mininimum size of this component.
     * @return A dimension object indicating this component's minimum size.
     * @see #getPreferredSize
     */	
    public Dimension getMinimumSize()
    {
        return new Dimension(p_width, p_height + tabHeight + 5);
    }

    public void setFont(Font font)
    {
        super.setFont(font);
        if(font != null)
            selectedTabFont = new Font(font.getName(), 1, font.getSize());
    }
    /**
    * Paints the TabPanel and its selected tab.
    */
    public void paint(Graphics g)
    {
        super.paint(g);
        FontMetrics fontmetrics = g.getFontMetrics(getFont());
        int fh = fontmetrics.getHeight();
        if(tabHeight < (fh + 6)){setTabHeight(fh+6);}        
        int i = tabHeight - 1;
        int j = 0;
        int k = getSize().height - 1;
        int l = (j + getSize().width) - 1;
        g.setColor(Color.darkGray);
        g.drawLine(j, k, l - 1, k);
        g.drawLine(l, i, l, k);
        g.setColor(Color.gray);
        g.drawLine(j + 1, k - 1, l - 2, k - 1);
        g.drawLine(l - 1, i + 1, l - 1, k - 1);
        g.setColor(Color.white);
        g.drawLine(j, i, j, k - 1);
        tabWidths = new int[labels.size()];
        int i1 = -1;
        int j1 = 0;
        int k1 = 2;
        for(int l1 = 0; l1 < labels.size(); l1++)
        {
            String s = (String)labels.elementAt(l1);
                tabWidths[l1] = fontmetrics.stringWidth(s) + tabWidthBuffer;
            if(labels.elementAt(l1).equals(selectedPanel))
            {
                i1 = l1;
                j1 = k1;
            }
            else
            {
                paintTab(g, false, k1, 0, tabWidths[l1], i, s/*, (Image)images.elementAt(l1)*/);
            }
            k1 += tabWidths[l1] - 1;
        }

        if(i1 > -1)
        {
            paintTab(g, true, j1, 0, tabWidths[i1], i, (String)labels.elementAt(i1)/*, (Image)images.elementAt(i1)*/);
            g.setColor(Color.white);
            g.drawLine(j, i, j1 - 2, i);
            g.drawLine(j1 + tabWidths[i1] + 2, i, l - 1, i);
            return;
        }
        else
        {
            g.setColor(Color.white);
            g.drawLine(j, i, l - 1, i);
            return;
        }
    }

    private void paintTab(Graphics g, boolean flag, int i, int j, int k, int l, String s)
    {
        int i1 = i;
        int j1 = j + 2;
        int k1 = (i + k) - 1;
        int l1 = (j + l) - 1;
        l -= 2;
        if(flag)
        {
            j1 -= 2;
            i1 -= 2;
            k1 += 2;
            l1++;
        }
        g.setColor(Color.darkGray);
        g.drawLine(k1 - 1, j1 + 2, k1 - 1, l1);
        g.drawRect(k1 - 2, j1 + 1, 0, 0);
        g.setColor(Color.gray);
        g.drawLine(k1 - 2, j1 + 2, k1 - 2, l1);
        g.setColor(Color.white);
        g.drawLine(i1, j1 + 2, i1, l1);
        g.drawLine(i1 + 2, j1, k1 - 3, j1);
        g.drawRect(i1 + 1, j1 + 1, 0, 0);
        g.setColor(Color.black);
        FontMetrics fontmetrics = g.getFontMetrics(getFont());
        g.drawString(s, i + (k - fontmetrics.stringWidth(s)) / 2, (fontmetrics.getHeight() - fontmetrics.getDescent()) + j1 + 3);
        if(flag)
        {
            g.setColor(getBackground());
            g.drawLine(i1 + 1, l1, k1 - 3, l1);
            g.drawLine(i1 + 1, j1 + 3, i1 + 1, l1);
        }
    }

    public void processMouseEvent(MouseEvent mouseevent)
    {
        int i = mouseevent.getX();
        int j = mouseevent.getY();
        if(mouseevent.getID() == 501 && j < tabHeight)
        {
            int k = 0;
            for(int l = 0; l < labels.size(); l++)
            {
                k += tabWidths[l];
                if(i < k)
                {
                    setPanel((String)labels.elementAt(l));
                    repaint();
                    return;
                }
            }

        }
        super.processMouseEvent(mouseevent);
    }

    protected Font selectedTabFont;
    /**
    * The tab labels.
    */
    protected Vector labels;
    /**
    * The panels representing each tab.
    */
    protected Vector panels;
    /**
    * The widths of the tabs.
    */
    protected int tabWidths[];
    /**
    * The label of the currently selected tab.
    */
    protected String selectedPanel;
    /**
    * The border within the FolderTabPanel to display between 
    * the rectangle and the contained components.
    */
    protected Insets insets;
    /**
    * The height of the tabs.
    */
    protected int tabHeight;
    /**
    * The margin to the left and right of the tab text.
    */
    protected int tabWidthBuffer;
    protected int p_width;
    protected int p_height;
}